#!/usr/bin/env bash
#
# corda-cli Bash Completion
# =======================
#
# Bash completion support for the `corda-cli` command,
# generated by [picocli](http://picocli.info/) version 4.5.2.
#
# Installation
# ------------
#
# 1. Source all completion scripts in your .bash_profile
#
#   cd $YOUR_APP_HOME/bin
#   for f in $(find . -name "*_completion"); do line=". $(pwd)/$f"; grep "$line" ~/.bash_profile || echo "$line" >> ~/.bash_profile; done
#
# 2. Open a new bash console, and type `corda-cli [TAB][TAB]`
#
# 1a. Alternatively, if you have [bash-completion](https://github.com/scop/bash-completion) installed:
#     Place this file in a `bash-completion.d` folder:
#
#   * /etc/bash-completion.d
#   * /usr/local/etc/bash-completion.d
#   * ~/bash-completion.d
#
# Documentation
# -------------
# The script is called by bash whenever [TAB] or [TAB][TAB] is pressed after
# 'corda-cli (..)'. By reading entered command line parameters,
# it determines possible bash completions and writes them to the COMPREPLY variable.
# Bash then completes the user input if only one entry is listed in the variable or
# shows the options if more than one is listed in COMPREPLY.
#
# References
# ----------
# [1] http://stackoverflow.com/a/12495480/1440785
# [2] http://tiswww.case.edu/php/chet/bash/FAQ
# [3] https://www.gnu.org/software/bash/manual/html_node/The-Shopt-Builtin.html
# [4] http://zsh.sourceforge.net/Doc/Release/Options.html#index-COMPLETE_005fALIASES
# [5] https://stackoverflow.com/questions/17042057/bash-check-element-in-array-for-elements-in-another-array/17042655#17042655
# [6] https://www.gnu.org/software/bash/manual/html_node/Programmable-Completion.html#Programmable-Completion
# [7] https://stackoverflow.com/questions/3249432/can-a-bash-tab-completion-script-be-used-in-zsh/27853970#27853970
#

if [ -n "$BASH_VERSION" ]; then
  # Enable programmable completion facilities when using bash (see [3])
  shopt -s progcomp
elif [ -n "$ZSH_VERSION" ]; then
  # Make alias a distinct command for completion purposes when using zsh (see [4])
  setopt COMPLETE_ALIASES
  alias compopt=complete

  # Enable bash completion in zsh (see [7])
  autoload -U +X compinit && compinit
  autoload -U +X bashcompinit && bashcompinit
fi

# CompWordsContainsArray takes an array and then checks
# if all elements of this array are in the global COMP_WORDS array.
#
# Returns zero (no error) if all elements of the array are in the COMP_WORDS array,
# otherwise returns 1 (error).
function CompWordsContainsArray() {
  declare -a localArray
  localArray=("$@")
  local findme
  for findme in "${localArray[@]}"; do
    if ElementNotInCompWords "$findme"; then return 1; fi
  done
  return 0
}
function ElementNotInCompWords() {
  local findme="$1"
  local element
  for element in "${COMP_WORDS[@]}"; do
    if [[ "$findme" = "$element" ]]; then return 1; fi
  done
  return 0
}

# The `currentPositionalIndex` function calculates the index of the current positional parameter.
#
# currentPositionalIndex takes three parameters:
# the command name,
# a space-separated string with the names of options that take a parameter, and
# a space-separated string with the names of boolean options (that don't take any params).
# When done, this function echos the current positional index to std_out.
#
# Example usage:
# local currIndex=$(currentPositionalIndex "mysubcommand" "$ARG_OPTS" "$FLAG_OPTS")
function currentPositionalIndex() {
  local commandName="$1"
  local optionsWithArgs="$2"
  local booleanOptions="$3"
  local previousWord
  local result=0

  for i in $(seq $((COMP_CWORD - 1)) -1 0); do
    previousWord=${COMP_WORDS[i]}
    if [ "${previousWord}" = "$commandName" ]; then
      break
    fi
    if [[ "${optionsWithArgs}" =~ ${previousWord} ]]; then
      ((result-=2)) # Arg option and its value not counted as positional param
    elif [[ "${booleanOptions}" =~ ${previousWord} ]]; then
      ((result-=1)) # Flag option itself not counted as positional param
    fi
    ((result++))
  done
  echo "$result"
}

# Bash completion entry point function.
# _complete_corda-cli finds which commands and subcommands have been specified
# on the command line and delegates to the appropriate function
# to generate possible options and subcommands for the last specified subcommand.
function _complete_corda-cli() {
  local cmds0=(network)
  local cmds1=(package)
  local cmds2=(pkg)
  local cmds3=(network configure)
  local cmds4=(network config)
  local cmds5=(network conf)
  local cmds6=(network list)
  local cmds7=(network ls)
  local cmds8=(network deploy)
  local cmds9=(network status)
  local cmds10=(network wait)
  local cmds11=(network terminate)
  local cmds12=(network restart)
  local cmds13=(package list)
  local cmds14=(package ls)
  local cmds15=(package depends)
  local cmds16=(package deps)
  local cmds17=(package cat)
  local cmds18=(package install)
  local cmds19=(package remove)
  local cmds20=(package rm)
  local cmds21=(pkg list)
  local cmds22=(pkg ls)
  local cmds23=(pkg depends)
  local cmds24=(pkg deps)
  local cmds25=(pkg cat)
  local cmds26=(pkg install)
  local cmds27=(pkg remove)
  local cmds28=(pkg rm)

  if CompWordsContainsArray "${cmds28[@]}"; then _picocli_corda-cli_pkg_rm; return $?; fi
  if CompWordsContainsArray "${cmds27[@]}"; then _picocli_corda-cli_pkg_remove; return $?; fi
  if CompWordsContainsArray "${cmds26[@]}"; then _picocli_corda-cli_pkg_install; return $?; fi
  if CompWordsContainsArray "${cmds25[@]}"; then _picocli_corda-cli_pkg_cat; return $?; fi
  if CompWordsContainsArray "${cmds24[@]}"; then _picocli_corda-cli_pkg_deps; return $?; fi
  if CompWordsContainsArray "${cmds23[@]}"; then _picocli_corda-cli_pkg_depends; return $?; fi
  if CompWordsContainsArray "${cmds22[@]}"; then _picocli_corda-cli_pkg_ls; return $?; fi
  if CompWordsContainsArray "${cmds21[@]}"; then _picocli_corda-cli_pkg_list; return $?; fi
  if CompWordsContainsArray "${cmds20[@]}"; then _picocli_corda-cli_package_rm; return $?; fi
  if CompWordsContainsArray "${cmds19[@]}"; then _picocli_corda-cli_package_remove; return $?; fi
  if CompWordsContainsArray "${cmds18[@]}"; then _picocli_corda-cli_package_install; return $?; fi
  if CompWordsContainsArray "${cmds17[@]}"; then _picocli_corda-cli_package_cat; return $?; fi
  if CompWordsContainsArray "${cmds16[@]}"; then _picocli_corda-cli_package_deps; return $?; fi
  if CompWordsContainsArray "${cmds15[@]}"; then _picocli_corda-cli_package_depends; return $?; fi
  if CompWordsContainsArray "${cmds14[@]}"; then _picocli_corda-cli_package_ls; return $?; fi
  if CompWordsContainsArray "${cmds13[@]}"; then _picocli_corda-cli_package_list; return $?; fi
  if CompWordsContainsArray "${cmds12[@]}"; then _picocli_corda-cli_network_restart; return $?; fi
  if CompWordsContainsArray "${cmds11[@]}"; then _picocli_corda-cli_network_terminate; return $?; fi
  if CompWordsContainsArray "${cmds10[@]}"; then _picocli_corda-cli_network_wait; return $?; fi
  if CompWordsContainsArray "${cmds9[@]}"; then _picocli_corda-cli_network_status; return $?; fi
  if CompWordsContainsArray "${cmds8[@]}"; then _picocli_corda-cli_network_deploy; return $?; fi
  if CompWordsContainsArray "${cmds7[@]}"; then _picocli_corda-cli_network_ls; return $?; fi
  if CompWordsContainsArray "${cmds6[@]}"; then _picocli_corda-cli_network_list; return $?; fi
  if CompWordsContainsArray "${cmds5[@]}"; then _picocli_corda-cli_network_conf; return $?; fi
  if CompWordsContainsArray "${cmds4[@]}"; then _picocli_corda-cli_network_config; return $?; fi
  if CompWordsContainsArray "${cmds3[@]}"; then _picocli_corda-cli_network_configure; return $?; fi
  if CompWordsContainsArray "${cmds2[@]}"; then _picocli_corda-cli_pkg; return $?; fi
  if CompWordsContainsArray "${cmds1[@]}"; then _picocli_corda-cli_package; return $?; fi
  if CompWordsContainsArray "${cmds0[@]}"; then _picocli_corda-cli_network; return $?; fi

  # No subcommands were specified; generate completions for the top-level command.
  _picocli_corda-cli; return $?;
}

# Generates completions for the options and subcommands of the `corda-cli` command.
function _picocli_corda-cli() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands="network package pkg"
  local flag_opts="--stacktrace -h --help --version -v"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `network` subcommand.
function _picocli_corda-cli_network() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands="configure config conf list ls deploy status wait terminate restart"
  local flag_opts="--stacktrace -h --help --version -v"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `package` subcommand.
function _picocli_corda-cli_package() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands="list ls depends deps cat install remove rm"
  local flag_opts="--stacktrace -h --help --version -v"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `pkg` subcommand.
function _picocli_corda-cli_pkg() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands="list ls depends deps cat install remove rm"
  local flag_opts="--stacktrace -h --help --version -v"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `configure` subcommand.
function _picocli_corda-cli_network_configure() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""
  local deploymentType_pos_param_args="docker-compose" # 0-0 values

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "configure" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      positionals=$( compgen -W "$deploymentType_pos_param_args" -- "${curr_word}" )
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `config` subcommand.
function _picocli_corda-cli_network_config() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""
  local deploymentType_pos_param_args="docker-compose" # 0-0 values

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "config" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      positionals=$( compgen -W "$deploymentType_pos_param_args" -- "${curr_word}" )
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `conf` subcommand.
function _picocli_corda-cli_network_conf() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""
  local deploymentType_pos_param_args="docker-compose" # 0-0 values

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "conf" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      positionals=$( compgen -W "$deploymentType_pos_param_args" -- "${curr_word}" )
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `list` subcommand.
function _picocli_corda-cli_network_list() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `ls` subcommand.
function _picocli_corda-cli_network_ls() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `deploy` subcommand.
function _picocli_corda-cli_network_deploy() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts="-n --network --networkName --file -f --port -p --container-tag -t --container-registry -r"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    --file|-f)
      compopt -o filenames
      COMPREPLY=( $( compgen -f -- "${curr_word}" ) ) # files
      return $?
      ;;
    --port|-p)
      return
      ;;
    --container-tag|-t)
      return
      ;;
    --container-registry|-r)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `status` subcommand.
function _picocli_corda-cli_network_status() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts="-n --network --networkName --format -f"
  local format_option_args="json pretty-json yaml text" # --format values

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    --format|-f)
      COMPREPLY=( $( compgen -W "${format_option_args}" -- "${curr_word}" ) )
      return $?
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `wait` subcommand.
function _picocli_corda-cli_network_wait() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --terminate"
  local arg_opts="-n --network --networkName --timeout -t"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    --timeout|-t)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `terminate` subcommand.
function _picocli_corda-cli_network_terminate() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --assume-yes -y --reset -r --forget -f"
  local arg_opts="-n --network --networkName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `restart` subcommand.
function _picocli_corda-cli_network_restart() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --run-migration -r"
  local arg_opts="-n --network --networkName -m --node --nodeName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    -m|--node|--nodeName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `list` subcommand.
function _picocli_corda-cli_package_list() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help -l"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "list" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `ls` subcommand.
function _picocli_corda-cli_package_ls() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help -l"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "ls" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `depends` subcommand.
function _picocli_corda-cli_package_depends() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "depends" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `deps` subcommand.
function _picocli_corda-cli_package_deps() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "deps" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `cat` subcommand.
function _picocli_corda-cli_package_cat() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `install` subcommand.
function _picocli_corda-cli_package_install() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --skip-notaries -s"
  local arg_opts="-n --network --networkName -m --node --nodeName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    -m|--node|--nodeName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "install" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 2147483647 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `remove` subcommand.
function _picocli_corda-cli_package_remove() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --assume-yes -y"
  local arg_opts="-n --network --networkName -m --node --nodeName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    -m|--node|--nodeName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `rm` subcommand.
function _picocli_corda-cli_package_rm() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --assume-yes -y"
  local arg_opts="-n --network --networkName -m --node --nodeName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    -m|--node|--nodeName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `list` subcommand.
function _picocli_corda-cli_pkg_list() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help -l"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "list" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `ls` subcommand.
function _picocli_corda-cli_pkg_ls() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help -l"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "ls" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `depends` subcommand.
function _picocli_corda-cli_pkg_depends() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "depends" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `deps` subcommand.
function _picocli_corda-cli_pkg_deps() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "deps" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 0 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `cat` subcommand.
function _picocli_corda-cli_pkg_cat() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}

  local commands=""
  local flag_opts="--stacktrace -h --help"
  local arg_opts=""

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `install` subcommand.
function _picocli_corda-cli_pkg_install() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --skip-notaries -s"
  local arg_opts="-n --network --networkName -m --node --nodeName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    -m|--node|--nodeName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    local currIndex
    currIndex=$(currentPositionalIndex "install" "${arg_opts}" "${flag_opts}")
    if (( currIndex >= 0 && currIndex <= 2147483647 )); then
      compopt -o filenames
      positionals=$( compgen -f -- "${curr_word}" ) # files
    fi
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `remove` subcommand.
function _picocli_corda-cli_pkg_remove() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --assume-yes -y"
  local arg_opts="-n --network --networkName -m --node --nodeName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    -m|--node|--nodeName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Generates completions for the options and subcommands of the `rm` subcommand.
function _picocli_corda-cli_pkg_rm() {
  # Get completion data
  local curr_word=${COMP_WORDS[COMP_CWORD]}
  local prev_word=${COMP_WORDS[COMP_CWORD-1]}

  local commands=""
  local flag_opts="--stacktrace -h --help --assume-yes -y"
  local arg_opts="-n --network --networkName -m --node --nodeName"

  compopt +o default

  case ${prev_word} in
    -n|--network|--networkName)
      return
      ;;
    -m|--node|--nodeName)
      return
      ;;
  esac

  if [[ "${curr_word}" == -* ]]; then
    COMPREPLY=( $(compgen -W "${flag_opts} ${arg_opts}" -- "${curr_word}") )
  else
    local positionals=""
    COMPREPLY=( $(compgen -W "${commands} ${positionals}" -- "${curr_word}") )
  fi
}

# Define a completion specification (a compspec) for the
# `corda-cli`, `corda-cli.sh`, and `corda-cli.bash` commands.
# Uses the bash `complete` builtin (see [6]) to specify that shell function
# `_complete_corda-cli` is responsible for generating possible completions for the
# current word on the command line.
# The `-o default` option means that if the function generated no matches, the
# default Bash completions and the Readline default filename completions are performed.
complete -F _complete_corda-cli -o default corda-cli corda-cli.sh corda-cli.bash
